@@ -66,7 +66,11 @@ SmokeDataFile::SmokeDataFile()
             declaredVirtualMethods[meth->getClass()] << meth;
         }
     }
-    
+
+    foreach (Type* type, usedTypes) {
+        insertTemplateParameters(*type);
+    }
+
     // if a class is used somewhere but not listed in the class list, mark it external
     for (QHash<QString, Class>::iterator iter = ::classes.begin(); iter != ::classes.end(); iter++) {
         if (iter.value().isTemplate() || Options::voidpTypes.contains(iter.key()))
@@ -96,6 +100,14 @@ SmokeDataFile::SmokeDataFile()
     }
 }
 
+void SmokeDataFile::insertTemplateParameters(const Type& type)
+{
+    foreach(const Type& t, type.templateArguments()) {
+        usedTypes << Type::registerType(t);
+        insertTemplateParameters(t);
+    }
+}
+
 bool SmokeDataFile::isClassUsed(const Class* klass)
 {
     for (QSet<Type*>::const_iterator it = usedTypes.constBegin(); it != usedTypes.constEnd(); it++) {
@@ -440,10 +452,14 @@ void SmokeDataFile::write()
             if (meth.parameters().size() > 0) {
                 outArgNames << ";";
                 for (int i = 0; i < meth.parameters().size(); i++) {
-                    QString paramName = meth.parameters()[i].name();
+                    Parameter parameter = meth.parameters()[i];
+                    QString paramName = parameter.name();
                     if (paramName == "") {
                         paramName = "arg" + QString::number(i + 1);
                     }
+                    if (!parameter.defaultValue().isEmpty()) {
+                        paramName += " = " + parameter.defaultValue();
+                    }
                     outArgNames << paramName;
                     if (i < meth.parameters().size() - 1) {
                         outArgNames << ",";
